(function () {
  function $ (selector) { return Array.prototype.slice.call(document.querySelectorAll(selector)); }
  function getDuration(elem) { return Math.max(750, (elem.getAttribute('data-carousel-duration') | 0) * 1000); }
  function loop(num, max) { return ((num % max) + max) % max; }

  Drupal.behaviors.paragraphDefaultsCarousel = {
    attach: function () {
      $('.carousel').forEach(function (frame) {
        if (!(frame.getAttribute('data-carousel-active') | 0)) {
          let start = Date.now(), paused = false, elapsed = 0, slide = 0, slidecount = 0, controls = [];

          function updateSlide(adj) {
            adj = adj | 0;
            slide += adj;

            slidecount = 0;
            for (let c = 0; c < frame.children.length; c++) {
              let elem = frame.children[c];

              if (elem.className.split(' ').indexOf('carousel-track') >= 0) {
                let actualIndex = loop(slide, elem.children.length);
                slidecount = Math.max(slidecount, elem.children.length);
                elem.style.transform = 'translateX(' + (actualIndex * -100) + '%)';
              }
            }

            if (slidecount > 1 && elem.hidden) {
              controls.forEach(function (elem) { elem.hidden = false; });
            } else if (slidecount <= 1 && !elem.hidden) {
              controls.forEach(function (elem) { elem.hidden = true; });
            }

            if (!paused) {
              start = Date.now();
              elapsed = 0;
            }
          }

          function advance() {
            if (!paused) {
              elapsed = Date.now() - start;
  
              if (elapsed >= getDuration(frame)) {
                updateSlide(1);
              }
            }
  
            window.requestAnimationFrame(advance);
          }
  
          window.requestAnimationFrame(advance);

          for (let c = 0; c < frame.children.length; c++) {
            let elem = frame.children[c];

            if (elem.className.split(' ').indexOf('carousel-controls') >= 0) {
              controls.push(elem);

              elem.addEventListener('click', function (e) {
                let button = e.target;

                switch (button.getAttribute('data-carousel-action')) {
                  case 'back':
                    updateSlide(-1);
                    break;
                  case 'forward':
                    updateSlide(1);
                    break;
                  case 'play':
                    paused = !paused;

                    if (paused) {
                      elapsed = Date.now() - start;
                      button.classList.remove('pause');
                      button.classList.add('play');
                    } else {
                      start = Date.now() - elapsed;
                      button.classList.remove('play');
                      button.classList.add('pause');
                    }

                    break;
                }
              });
            } else if (elem.className.split(' ').indexOf('carousel-track') >= 0) {
              slidecount = Math.max(slidecount, elem.children.length);
            }
          }

          if (slidecount > 1 && elem.hidden) {
            controls.forEach(function (elem) { elem.hidden = false; });
          } else if (slidecount <= 1 && !elem.hidden) {
            controls.forEach(function (elem) { elem.hidden = true; });
          }

          frame.setAttribute('data-carousel-active', 1);
        }
      });
    }
  };
})();
